/******************************************************************************
 * Copyright (c) 1998 by Mindbright Technology AB, Stockholm, Sweden.
 *
 *                        All Rights Reserved
 *
 *               www.mindbright.se, info@mindbright.se
 *
 * This program and its source-code may be used FREE of charge for strictly
 * NON-COMMERCIAL purposes. Companies or persons wishing to make non-commercial
 * software that in whole or in part contains or is derived from this program
 * or any part thereof must inform the original author of such work. The
 * original author must also be given attribution as the author of used parts.
 * For COMMERCIAL use all parts of this program originally developed by
 * Mindbright Technology must be LICENSED under Mindbright Technology's
 * SHAREWARE license, please contact Mindbright Technology for information on
 * this. This copyright notice must allways be included and unchanged.
 *
 * IF LICENSED FREE OF CHARGE AND/OR IF NOT STATED OTHERWISE THIS PROGRAM COMES
 * WITH NO WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, IN NO EVENT SHALL
 * MINDBRIGHT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *****************************************************************************/
package mindbright.ssh;

import java.net.*;
import java.io.*;
import java.util.Hashtable;
import java.util.Vector;

public class SSHConnectChannel extends SSHTxChannel {
  SSHChannelController controller;

  Hashtable hostMap;

  public SSHConnectChannel(SSHChannelController controller) {
    super(null, SSH.CONNECT_CHAN_NUM);
    this.controller     = controller; 
    this.hostMap        = new Hashtable();
  }

  public synchronized void addHostMapPermanent(String fromHost, String toHost, int toPort) {
    Vector hostPortPair = new Vector();
    hostPortPair.addElement(toHost);
    hostPortPair.addElement(new Integer(toPort));
    hostPortPair.addElement(new Boolean(true));
    hostMap.put(fromHost, hostPortPair);
  }
  public synchronized void addHostMapTemporary(String fromHost, String toHost, int toPort) {
    Vector hostPortPair = new Vector();

    // !!!
    System.out.println("map add: " + fromHost + " to " + toHost + ":" + toPort);

    hostPortPair.addElement(toHost);
    hostPortPair.addElement(new Integer(toPort));
    hostPortPair.addElement(new Boolean(false));
    hostMap.put(fromHost, hostPortPair);
  }

  public synchronized void delHostMap(String fromHost) {
    hostMap.remove(fromHost);
  }

  public synchronized Vector getHostMap(String fromHost) {
    Vector hostPortPair = (Vector)hostMap.get(fromHost);
    if(hostPortPair != null && !(((Boolean)hostPortPair.elementAt(2)).booleanValue())) {
      delHostMap(fromHost);
    }
    return hostPortPair;
  }

  public void serviceLoop() throws Exception {
    SSHPduInputStream inPdu;
    int               remoteChannel;
    int               port;
    String            host;
    String            origin;
    Socket            fwdSocket;

    for(;;) {
      inPdu         = (SSHPduInputStream) queue.getFirst();
      remoteChannel = inPdu.readInt();
      host          = inPdu.readString();
      port          = inPdu.readInt();


      if(controller.haveHostInFwdOpen())
	origin = inPdu.readString();
      else
	origin = "unknown (origin-option not used)";

      // See if there is a translation entry for this host
      //
      Vector hostPortPair = getHostMap(host);
      if(hostPortPair != null) {
	// !!!
	System.out.print("mapping: " + host);

	host = (String)hostPortPair.elementAt(0);
	port = ((Integer)hostPortPair.elementAt(1)).intValue();

	// !!!
	System.out.println(" to " + host + ":" + port);

      }

      SSHPduOutputStream respPdu;

      try {
	fwdSocket        = new Socket(host, port);
	int newChan      = controller.newChannelId();
	SSHTunnel tunnel = new SSHTunnel(fwdSocket, newChan, remoteChannel, controller);
	controller.addTunnel(tunnel);
	respPdu = new SSHPduOutputStream(SSH.MSG_CHANNEL_OPEN_CONFIRMATION, controller.sndCipher);
	respPdu.writeInt(remoteChannel);
	respPdu.writeInt(newChan);

	System.out.println("Port open (" + origin + ") : " + host + ": " + port +
			   " (#" + remoteChannel + ")" + " new: " + newChan);

	controller.transmit(respPdu);

	// We wait until after we have put the response in the
	// controllers tx-queue with starting the tunnel
	// (to avoid data reaching the server before the response)
	//
	tunnel.start();

      } catch (IOException e) {
 	respPdu = new SSHPduOutputStream(SSH.MSG_CHANNEL_OPEN_FAILURE, controller.sndCipher);
 	respPdu.writeInt(remoteChannel);

	System.out.println("Failed port open (" + origin + ") : " + host + ": " + port +
			   " (#" + remoteChannel + ")");

	controller.transmit(respPdu);
      }

    }
  }

}
