/******************************************************************************
 * Copyright (c) 1998 by Mindbright Technology AB, Stockholm, Sweden.
 *
 *                        All Rights Reserved
 *
 *               www.mindbright.se, info@mindbright.se
 *
 * This program and its source-code may be used FREE of charge for strictly
 * NON-COMMERCIAL purposes. Companies or persons wishing to make non-commercial
 * software that in whole or in part contains or is derived from this program
 * or any part thereof must inform the original author of such work. The
 * original author must also be given attribution as the author of used parts.
 * For COMMERCIAL use all parts of this program originally developed by
 * Mindbright Technology must be LICENSED under Mindbright Technology's
 * SHAREWARE license, please contact Mindbright Technology for information on
 * this. This copyright notice must allways be included and unchanged.
 *
 * IF LICENSED FREE OF CHARGE AND/OR IF NOT STATED OTHERWISE THIS PROGRAM COMES
 * WITH NO WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, IN NO EVENT SHALL
 * MINDBRIGHT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *****************************************************************************/
package mindbright.ssh;

import java.util.Vector;

public class SSHPduQueue {

  Vector     queue;
  SSHChannel owner;
  boolean    isWaiting;

  public SSHPduQueue(SSHChannel owner) {
    this.queue     = new Vector();
    this.owner     = owner;
    this.isWaiting = false;
  }

  public void addLast(SSHPdu pdu) {
    synchronized(queue) {
      queue.addElement(pdu);
      if(isWaiting)
	queue.notify();
    }
  }

  public void addFirst(SSHPdu pdu) {
    synchronized(queue) {
      queue.insertElementAt(pdu, 0);
      if(isWaiting)
	queue.notify();
    }
  }

  public void release() {
    synchronized(queue) {
      if(isWaiting)
	queue.notify();
    }
  }

  public boolean isEmpty() {
    boolean isEmpty;
    synchronized(queue) {
      isEmpty = queue.isEmpty();
    }
    return isEmpty;
  }

  public SSHPdu getFirst() {
    boolean doSleep = false;
    SSHPdu  pdu     = null;

    synchronized(queue) {
      if(queue.isEmpty()) {
	isWaiting = true;
	try {
	  queue.wait();
	} catch (InterruptedException e) {
	  // !!!
	}
      }
      isWaiting = false;
      try {
	pdu = (SSHPdu) queue.firstElement();
	queue.removeElementAt(0);
      } catch (Exception e) {
	pdu = null;
      }
    }

    return pdu;
  }

}
