/******************************************************************************
 * Copyright (c) 1998 by Mindbright Technology AB, Stockholm, Sweden.
 *
 *                        All Rights Reserved
 *
 *               www.mindbright.se, info@mindbright.se
 *
 * This program and its source-code may be used FREE of charge for strictly
 * NON-COMMERCIAL purposes. Companies or persons wishing to make non-commercial
 * software that in whole or in part contains or is derived from this program
 * or any part thereof must inform the original author of such work. The
 * original author must also be given attribution as the author of used parts.
 * For COMMERCIAL use all parts of this program originally developed by
 * Mindbright Technology must be LICENSED under Mindbright Technology's
 * SHAREWARE license, please contact Mindbright Technology for information on
 * this. This copyright notice must allways be included and unchanged.
 *
 * IF LICENSED FREE OF CHARGE AND/OR IF NOT STATED OTHERWISE THIS PROGRAM COMES
 * WITH NO WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, IN NO EVENT SHALL
 * MINDBRIGHT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *****************************************************************************/
package mindbright.terminal;

public interface Terminal {

  public String terminalType();
  public int    rows();
  public int    cols();
  public int    vpixels();
  public int    hpixels();

  public void write(char c);
  public void write(char[] c, int off, int len);
  public void write(String str);

  public void addTerminalListener(TerminalListener listener);

  public void sendBytes(byte[] b);

  public void doBell();
  public void doBS();
  public void doTab();
  public void doCR();
  public void doLF();

  public void    setReverseVideo(boolean val);
  public boolean getReverseVideo();
  public void    setAutoWrap(boolean val);
  public boolean getAutoWrap();

  public void    setCursorKeysMode(boolean val);

  public void resetWindow();
  public void setWindow(int top, int bottom);
  public void setWindow(int top, int right, int bottom, int left);
  public int  getWindowTop();
  public int  getWindowBottom();
  public int  getWindowLeft();
  public int  getWindowRight();

  public int getCursorV();
  public int getCursorH();

  public void cursorSetPos(int v, int h, boolean relative);
  public void cursorUp(int n);
  public void cursorDown(int n);
  public void cursorForward(int n);
  public void cursorBackward(int n);
  public void cursorIndex(int n);
  public void cursorIndexRev(int n);

  public void cursorSave();
  public void cursorRestore();

  public void scrollUp(int n);
  public void scrollDown(int n);

  public void clearBelow();
  public void clearAbove();
  public void clearScreen();
  public void clearRight();
  public void clearLeft();
  public void clearLine();

  public void insertChars(int n);
  public void insertLines(int n);
  public void deleteChars(int n);
  public void deleteLines(int n);

  public void clearAllAttributes();
  public void setBold(boolean val);
  public void setLowIntensity(boolean val);
  public void setUnderline(boolean val);
  public void setHilite(boolean val);
  public void setInverseVideo(boolean val);
  public void setInvisible(boolean val);

  public void setForegroundColor(int c);
  public void setBackgroundColor(int c);

}
